@@ -101,7 +101,21 @@ static const char *role_names [] =
   "fooler",
   "paragraph",
   "ruler",
-  "application"
+  "application",
+  "autocomplete",
+  "editbar",
+  "embedded",
+  "entry",
+  "chart",
+  "caption",
+  "document_frame",
+  "heading",
+  "page",
+  "section",
+  "form",
+  "redundant object",
+  "link",
+  "input method window"
 };
 
 #define MAX_ROLES (sizeof (role_names) / sizeof (char *))
@@ -191,6 +205,17 @@ cspi_init_role_table (AccessibleRole *role_table)
   role_table [Accessibility_ROLE_AUTOCOMPLETE] = SPI_ROLE_AUTOCOMPLETE;
   role_table [Accessibility_ROLE_EDITBAR] = SPI_ROLE_EDITBAR;
   role_table [Accessibility_ROLE_EMBEDDED] = SPI_ROLE_EMBEDDED;
+  role_table [Accessibility_ROLE_ENTRY] = SPI_ROLE_ENTRY;
+  role_table [Accessibility_ROLE_CHART] = SPI_ROLE_CHART;
+  role_table [Accessibility_ROLE_CAPTION] = SPI_ROLE_CAPTION;
+  role_table [Accessibility_ROLE_DOCUMENT_FRAME] = SPI_ROLE_DOCUMENT_FRAME;
+  role_table [Accessibility_ROLE_HEADING] = SPI_ROLE_HEADING;
+  role_table [Accessibility_ROLE_PAGE] = SPI_ROLE_PAGE;
+  role_table [Accessibility_ROLE_SECTION] = SPI_ROLE_SECTION;
+  role_table [Accessibility_ROLE_FORM] = SPI_ROLE_FORM;
+  role_table [Accessibility_ROLE_REDUNDANT_OBJECT] = SPI_ROLE_REDUNDANT_OBJECT;
+  role_table [Accessibility_ROLE_LINK] = SPI_ROLE_LINK;
+  role_table [Accessibility_ROLE_INPUT_METHOD_WINDOW] = SPI_ROLE_INPUT_METHOD_WINDOW;
 
   return TRUE;
 }
@@ -218,13 +243,13 @@ cspi_role_from_spi_role (Accessibility_Role role)
 }
 
 AccessibleAttributeSet *
-cspi_attribute_set_from_sequence (const Accessibility_AttributeSet *seq)
+_cspi_attribute_set_from_sequence (const Accessibility_AttributeSet *seq)
 {
     AccessibleAttributeSet *set = g_new0 (AccessibleAttributeSet, 1);
     int i;
 
     set->len = seq->_length;
-    set->attributes = g_newa (char *, set->len);
+    set->attributes = g_new0 (char *, set->len);
     for (i = 0; i < set->len; ++i)
     {
 	set->attributes[i] = g_strdup (seq->_buffer [i]);
@@ -525,6 +550,8 @@ Accessible_getRoleName (Accessible *obj)
  * This method will return useful values for roles that fall outside the
  * enumeration used in Accessible_getRole ().
  *
+ * @Since: AT-SPI 1.4
+ *
  * Returns: a UTF-8 string specifying the role of this #Accessible object.
  *
  **/
@@ -547,6 +574,8 @@ Accessible_getLocalizedRoleName (Accessible *obj)
  * Accessible_getStateSet:
  * @obj: a pointer to the #Accessible object on which to operate.
  *
+ * Gets the current state of an object.
+ *
  * Returns: a pointer to an #AccessibleStateSet representing the object's current state.
  **/
 AccessibleStateSet *
@@ -577,11 +606,13 @@ Accessible_getStateSet (Accessible *obj)
 
 /**
  * Accessible_getAttributes:
+ * @obj: The #Accessible being queried.
+ *
  * Get the #AttributeSet representing any assigned 
  * name-value pair attributes or annotations for this object.
  * For typographic, textual, or textually-semantic attributes, see
  * AccessibleText_getAttributes instead.
- * @obj: The #Accessible being queried.
+ *
  * Returns: The name-value-pair attributes assigned to this object.
  */
 AccessibleAttributeSet *
@@ -596,16 +627,18 @@ Accessible_getAttributes (Accessible *obj)
 	CSPI_OBJREF (obj), cspi_ev ());
     cspi_return_val_if_ev ("getAttributes", NULL);
     
-    retval = cspi_attribute_set_from_sequence (corba_seq);
+    retval = _cspi_attribute_set_from_sequence (corba_seq);
     CORBA_free (corba_seq);
 
-    return NULL;
+    return retval;
 }
 
 /**
  * Accessible_getHostApplication:
- * Get the containing #AccessibleApplication for an object.
  * @obj: The #Accessible being queried.
+ *
+ * Get the containing #AccessibleApplication for an object.
+ *
  * Returns: the containing AccessibleApplication instance for this object.
  */
 AccessibleApplication *
@@ -657,6 +690,21 @@ Accessible_isApplication (Accessible *obj)
 			      "IDL:Accessibility/Application:1.0");
 }
 
+/**                      
+ * Accessible_isCollection:                                                                                                                                                                          * @obj: a pointer to the #Accessible instance to query.                                                                                                                                          
+ *                          
+ * Query whether the specified #Accessible implements #AccessibleCollection.    
+ * Returns: #TRUE if @obj implements the #AccessibleCollection interface,                                                                                                               
+ *          #FALSE otherwise.
+ **/
+
+SPIBoolean
+Accessible_isCollection (Accessible *obj)
+{
+     return cspi_accessible_is_a (obj,
+				  "IDL:Accessibility/Collection:1.0");
+}
+
 /**
  * Accessible_isComponent:
  * @obj: a pointer to the #Accessible instance to query.
@@ -673,6 +721,22 @@ Accessible_isComponent (Accessible *obj)
 			      "IDL:Accessibility/Component:1.0");
 }
 
+/**
+ * Accessible_isDocument:
+ * @obj: a pointer to the #Accessible instance to query.
+ *
+ * Query whether the specified #Accessible implements #AccessibleDocument.
+ *
+ * Returns: #TRUE if @obj implements the #AccessibleDocument interface,
+ *          #FALSE otherwise.
+ **/
+SPIBoolean
+Accessible_isDocument (Accessible *obj)
+{
+  return cspi_accessible_is_a (obj,
+			      "IDL:Accessibility/Document:1.0");
+}
+
 /**
  * Accessible_isEditableText:
  * @obj: a pointer to the #Accessible instance to query.
@@ -688,6 +752,22 @@ Accessible_isEditableText (Accessible *obj)
   return cspi_accessible_is_a (obj,
 			      "IDL:Accessibility/EditableText:1.0");
 }
+                                                                                                                                                                        
+/**
+ * Accessible_isMatchRule:
+ * @obj: a pointer to the #Accessible instance to query.
+ *
+ * Query whether the specified #Accessible implements #AccessibleMatchRule.
+ *
+ * Returns: #TRUE if @obj implements the #AccessibleMatchRule interface,
+ *          #FALSE otherwise.
+ **/
+SPIBoolean
+Accessible_isMatchRule (Accessible *obj)
+{
+     return cspi_accessible_is_a (obj, 
+				  "IDL:Accessibility/MatchRule:1.0");
+}
 
 /**
  * Accessible_isHypertext:
@@ -834,6 +914,23 @@ Accessible_getAction (Accessible *obj)
 	  obj, "IDL:Accessibility/Action:1.0");
 }
 
+
+/**
+ * Accessible_getCollection:
+ * @obj: a pointer to the #Accessible instance to query.
+ *
+ * Get the #AccessibleCollection interface for an #Accessible.
+ *
+ * Returns: a pointer to an #AccessibleCollection interface instance, or
+ *          NULL if @obj does not implement #AccessibleCollection.
+ **/
+AccessibleCollection *
+Accessible_getCollection (Accessible *obj)
+{
+  return (AccessibleCollection *) Accessible_queryInterface (
+	  obj, "IDL:Accessibility/Collection:1.0");
+}
+
 /**
  * Accessible_getComponent:
  * @obj: a pointer to the #Accessible instance to query.
@@ -849,6 +946,21 @@ Accessible_getComponent (Accessible *obj)
   return (AccessibleComponent *) Accessible_queryInterface (
 	  obj, "IDL:Accessibility/Component:1.0");
 }
+/**
+ * Accessible_getDocument:
+ * @obj: a pointer to the #Accessible instance to query.
+ *
+ * Get the #AccessibleDocument interface for an #Accessible.
+ *
+ * Returns: a pointer to an #AccessibleDocument interface instance, or
+ *          NULL if @obj does not implement #AccessibleDocument.
+ **/
+AccessibleDocument *
+Accessible_getDocument (Accessible *obj)
+{
+  return (AccessibleDocument *) Accessible_queryInterface (
+	  obj, "IDL:Accessibility/Document:1.0");
+}
 
 /**
  * Accessible_getEditableText:
@@ -902,7 +1014,21 @@ Accessible_getImage (Accessible *obj)
 	  obj, "IDL:Accessibility/Image:1.0");
 }
 
-
+/**
+ * Accessible_getMatchRule:
+ * @obj: a pointer to the #Accessible instance to query.
+ *
+ * Get the #AccessibleMatchRule interface for an #Accessible.
+ *
+ * Returns: a pointer to an #AccessibleMatchRule interface instance, or
+ *          NULL if @obj does not implement #AccessibleMatchRule.
+ **/
+AccessibleMatchRule *
+Accessible_getMatchRule (Accessible *obj)
+{
+  return (AccessibleMatchRule *) Accessible_queryInterface (
+	  obj, "IDL:Accessibility/MatchRule:1.0");
+}
 
 /**
  * Accessible_getSelection:
@@ -1081,6 +1207,9 @@ cspi_init_relation_type_table (AccessibleRelationType *relation_type_table)
   relation_type_table [Accessibility_RELATION_EMBEDS] = SPI_RELATION_EMBEDS;
   relation_type_table [Accessibility_RELATION_EMBEDDED_BY] = SPI_RELATION_EMBEDDED_BY;
   relation_type_table [Accessibility_RELATION_POPUP_FOR] = SPI_RELATION_POPUP_FOR;
+  relation_type_table [Accessibility_RELATION_PARENT_WINDOW_OF] = SPI_RELATION_PARENT_WINDOW_OF;
+  relation_type_table [Accessibility_RELATION_DESCRIBED_BY] = SPI_RELATION_DESCRIBED_BY;
+  relation_type_table [Accessibility_RELATION_DESCRIPTION_FOR] = SPI_RELATION_DESCRIPTION_FOR;
   return TRUE;
 }
 
@@ -1242,6 +1371,13 @@ spi_state_to_corba (AccessibleState state)
       MAP_STATE (VISIBLE);
       MAP_STATE (MANAGES_DESCENDANTS);
       MAP_STATE (INDETERMINATE);
+      MAP_STATE (TRUNCATED);
+      MAP_STATE (REQUIRED);
+      MAP_STATE (INVALID_ENTRY);
+      MAP_STATE (SUPPORTS_AUTOCOMPLETION);
+      MAP_STATE (SELECTABLE_TEXT);
+      MAP_STATE (IS_DEFAULT);
+      MAP_STATE (VISITED);
     default:
       return Accessibility_STATE_INVALID;
   }
